@@ -23,7 +23,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: page-info-dialog.c,v 1.54 2006/12/17 18:53:28 jframeau Exp $
+ *  $Id: page-info-dialog.c 1456 2007-03-02 23:39:11Z bberg $
  */
 
 #include "config.h"
@@ -942,6 +942,9 @@ mozilla_embed_type_to_string (EmbedPageMediumType type)
 		case MEDIUM_IMAGE: 
 			s_type = _("Image");
 			break;
+		case MEDIUM_BG_IMAGE: 
+			s_type = _("Background image");
+			break;
 		case MEDIUM_EMBED:
 			s_type = _("Embed");
 			break;
@@ -963,6 +966,7 @@ mozilla_embed_type_to_string (EmbedPageMediumType type)
 
 static gboolean
 media_is_medium_blocked (EphyAdBlockManager *manager,
+		         EphyEmbed *embed,
 		         const char *url,
 			 EmbedPageMediumType type)
 {
@@ -974,6 +978,7 @@ media_is_medium_blocked (EphyAdBlockManager *manager,
 		case MEDIUM_ICON:
 			should_load = ephy_adblock_manager_should_load (
 							manager, 
+							embed,
 					 		url, 
 							AD_URI_CHECK_TYPE_IMAGE);
 
@@ -983,6 +988,7 @@ media_is_medium_blocked (EphyAdBlockManager *manager,
 		case MEDIUM_APPLET:
 			should_load = ephy_adblock_manager_should_load (
 							manager, 
+							embed,
 					 		url, 
 							AD_URI_CHECK_TYPE_OBJECT);
 
@@ -990,6 +996,7 @@ media_is_medium_blocked (EphyAdBlockManager *manager,
 		default:
 			should_load = ephy_adblock_manager_should_load (
 							manager, 
+							embed,
 					 		url, 
 							AD_URI_CHECK_TYPE_OTHER);
 	}
@@ -1000,6 +1007,8 @@ static void
 media_sync_adblock (EphyAdBlockManager *manager, 
 		    TreeviewInfoPage *page)
 {
+	PageInfoDialog *dialog = page->page.dialog;
+	EphyEmbed *embed = dialog->priv->embed;
 	GtkTreeIter   iter;
 	gboolean      iter_valid;
 	GtkTreeModel *model = GTK_TREE_MODEL (page->store);
@@ -1014,7 +1023,7 @@ media_sync_adblock (EphyAdBlockManager *manager,
 
 		gtk_tree_model_get (model, &iter, COL_MEDIUM_URL, &url, COL_MEDIUM_TYPE, &type, -1);
 
-		should_load = media_is_medium_blocked (manager, url, type);
+		should_load = media_is_medium_blocked (manager, embed, url, type);
 
 		gtk_list_store_set (page->store, &iter, COL_MEDIUM_BLOCKED, !should_load, -1);
 		
@@ -1194,7 +1203,20 @@ media_treeview_selection_changed_cb (GtkTreeSelection *selection,
 
 		if (url != NULL && !media_is_embedded_medium (type))
 		{
-			ephy_embed_load_url (page->embed, url);
+			/* Workaround for bad content type */
+			char *data = 
+				g_markup_printf_escaped ("data:text/html,"
+							 "<html><body><img src="
+					     		 "%s></body></html>", url);
+							
+			/* We ask explicitly to not download anything. */
+			g_object_set_data (G_OBJECT (page->embed), 
+					   "content-handler-deny", 
+					   GINT_TO_POINTER (TRUE));
+
+			ephy_embed_load_url (page->embed, data);
+
+			g_free (data);
 		}
 		else
 		{
@@ -1471,6 +1493,7 @@ media_info_page_fill (InfoPage *ipage)
 {
 	TreeviewInfoPage *tpage = (TreeviewInfoPage *) ipage;
 	PageInfoDialog *dialog = ipage->dialog;
+	EphyEmbed *embed = dialog->priv->embed;
 	GtkListStore *store = tpage->store;
 	GtkTreeIter iter;
 	GList *media, *l;	
@@ -1489,6 +1512,7 @@ media_info_page_fill (InfoPage *ipage)
 		EmbedPageMedium *media = (EmbedPageMedium *) l->data;
 
 		should_load = media_is_medium_blocked (adblock_manager, 
+						       embed, 
 						       media->url, 
 						       media->type);
 
@@ -1504,6 +1528,12 @@ media_info_page_fill (InfoPage *ipage)
 				    COL_MEDIUM_TYPE, media->type,
 				    -1);
 	}
+	if (ephy_adblock_manager_has_blocker (adblock_manager))
+	{
+		gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(store), 
+						      COL_MEDIUM_BLOCKED, 	
+						      GTK_SORT_DESCENDING);
+	}
 }
 
 static InfoPage *
@@ -1585,9 +1615,11 @@ links_drag_data_get_cb (GtkWidget *widget,
 
 static gboolean
 links_is_link_blocked (EphyAdBlockManager *manager,
+		       EphyEmbed *embed,
 		       const char *url)
 {
 	return ephy_adblock_manager_should_load (manager, 
+						 embed,
 						 url, 
 						 AD_URI_CHECK_TYPE_OTHER);
 }
@@ -1596,6 +1628,8 @@ static void
 links_sync_adblock (EphyAdBlockManager *manager, 
 		    TreeviewInfoPage *page)
 {
+	PageInfoDialog *dialog = page->page.dialog;
+	EphyEmbed *embed = dialog->priv->embed;
 	GtkTreeIter   iter;
 	gboolean      iter_valid;
 	GtkTreeModel *model = GTK_TREE_MODEL (page->store);
@@ -1609,7 +1643,7 @@ links_sync_adblock (EphyAdBlockManager *manager,
 
 		gtk_tree_model_get (model, &iter, COL_LINK_URL, &url, -1);
 
-		should_load = links_is_link_blocked (manager, url);
+		should_load = links_is_link_blocked (manager, embed, url);
 
 		gtk_list_store_set (page->store, &iter, COL_LINK_BLOCKED, !should_load, -1);
 		
@@ -1727,6 +1761,7 @@ links_info_page_fill (InfoPage *ipage)
 {
 	TreeviewInfoPage *tpage = (TreeviewInfoPage *) ipage;
 	PageInfoDialog *dialog = ipage->dialog;
+	EphyEmbed *embed = dialog->priv->embed;
 	GtkListStore *store = tpage->store;
 	GtkTreeIter iter;
 	GList *links, *l;
@@ -1745,6 +1780,7 @@ links_info_page_fill (InfoPage *ipage)
 		EmbedPageLink *link = (EmbedPageLink *) l->data;
 
 		should_load = links_is_link_blocked (adblock_manager, 
+						     embed,
 						     link->url);
 
 		gtk_list_store_append (store, &iter);
@@ -1755,6 +1791,12 @@ links_info_page_fill (InfoPage *ipage)
 				    COL_LINK_REL, link->rel,
 				    -1);
 	}
+	if (ephy_adblock_manager_has_blocker (adblock_manager))
+	{
+		gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(store), 
+						      COL_MEDIUM_BLOCKED, 	
+						      GTK_SORT_DESCENDING);
+	}
 }
 
 static void
